#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <mpi.h>
#include <omp.h>
#include <signal.h>
#include <unistd.h>

// Skeleton for the problem of A puzzle of minimum squares
// Spanish Parallel Programming Contest 2012, problem D

void writepieces(int p, int **pieces){
  int i, j;

  for(i=0;i<p;i++)
  {
     printf("%d: ",pieces[i][0]);
     for(j=1;j<2*pieces[i][0]-1;j+=2)
     {
        printf("(%d,%d) ",pieces[i][j],pieces[i][j+1]);
      }
      printf("\n");
   }
   printf("\n");
}

/*
c
c     mseconds - returns elapsed milliseconds since Jan 1st, 1970.
c
*/
long long mseconds(void){
  struct timeval t;
  gettimeofday(&t, NULL);
  return t.tv_sec*1000 + t.tv_usec/1000;
}

static void alarm_handler(int sig) {
  fprintf(stderr, "Time Limit Exceeded\n");
  abort();
}


extern int sec(int,int,int **,int,int);


int main(int argc,char *argv[]) {
  int node,np,i,j,k,n,p,temp,nproblems,**pieces,minimum;
  long long ti,tf,tt=0;

  FILE *stats_file = fopen("stats", "w");

  struct sigaction sact;
  sigemptyset(&sact.sa_mask);
  sact.sa_flags = 0;
  sact.sa_handler = alarm_handler;
  sigaction(SIGALRM, &sact, NULL);
  alarm(30);  /* time limit */
 
  MPI_Init(&argc,&argv);
  MPI_Comm_size(MPI_COMM_WORLD,&np);
  MPI_Comm_rank(MPI_COMM_WORLD,&node);

// The number of test cases is read
// and sent to all the processes
  if(node==0)
  {
    scanf("%d",&nproblems);
    MPI_Bcast(&nproblems,1,MPI_INT,0,MPI_COMM_WORLD);
  }
  else
  {
    MPI_Bcast(&nproblems,1,MPI_INT,0,MPI_COMM_WORLD);
  }
  for(i=0;i<nproblems;i++)
  {
    if(node==0)
    {
      scanf("%d",&n);		  // The first argument is the checkerboard size
      scanf("%d",&p);                   // The second argument is the number of pieces
      // Space for the pieces
      pieces = (int **) malloc(sizeof(int*)*p);
      for(j=0;j<p;j++)
      {
         scanf("%d",&temp);
         pieces[j]=(int *) malloc(sizeof(int)*(2*temp-1));
         pieces[j][0]=temp;
         for(k=1;k<2*temp-1;k++)
         {
            scanf("%d",&(pieces[j][k]));
          }
       }
    }
    else
    {
       pieces=NULL;
    }
    MPI_Bcast(&n,1,MPI_INT,0,MPI_COMM_WORLD);
    MPI_Bcast(&p,1,MPI_INT,0,MPI_COMM_WORLD);

#ifdef DEBUG
// While debugging the pieces are written
     writepieces(p,pieces);
#endif
      
    MPI_Barrier(MPI_COMM_WORLD);
    ti=mseconds(); 
    minimum=sec(n,p,pieces,node,np);
    MPI_Barrier(MPI_COMM_WORLD);
    tf=mseconds(); 
    if(node==0)
    {

// The time of the first problem is not considered
      if(i!=0) tt+=tf-ti;
#ifdef TIME
// to obtain the execution time of each running
      fprintf(stats_file, "%Ld\n", tf-ti);
#endif
// The results of each problem are written,
// even for the first problem
      printf("%d\n",minimum);
       for(j=0;j<p;j++)
      {
         free(pieces[j]);
       }
       free(pieces);
    }
  }
  
  if(node==0)
  { 
    fprintf(stats_file, "%Ld\n", tt);
    fclose(stats_file);
  }
  MPI_Finalize();
  return 0;
}
