/*
  CPP_CONTEST=2012
  CPP_PROBLEM=D
  CPP_LANG=C+OPENMP+MPI
  CPP_NUM_CORES=1
  CPP_PROCESSES_PER_NODE=1
 */


#include <stdlib.h>
#include <stdio.h>

void removepiece(int *piece,int level,int *solution,int n,int *cb)
{
  int i;

  cb[solution[level]]=0;
  for(i=1;i<(2*piece[0]-2);i+=2)
  {
    cb[(solution[level]/n+piece[i])*n+solution[level]%n+piece[i+1]]=0;
  }
}
  
void putpiece(int *piece,int level,int *solution,int n,int *cb)
{
  int i,ident=level+1;

  cb[solution[level]]=ident;
  for(i=1;i<(2*piece[0]-2);i+=2)
  {
    cb[(solution[level]/n+piece[i])*n+solution[level]%n+piece[i+1]]=ident;
  }
}

int feasiblepiece(int *piece,int level,int *solution,int n,int *cb)
{
  int i,valid=1,fil,col;

  if(cb[solution[level]]!=0)
    return 0;
  i=1;
  while(i<(2*piece[0]-2) && valid)
  {
    fil=solution[level]/n+piece[i];
    if(fil<0 || fil>(n-1))
    {
      valid=0;
    }
    else
    {
      col=solution[level]%n+piece[i+1];
      if(col<0 || col>(n-1))
      {
        valid=0;
      }
      else
      {
        if(cb[fil*n+col]!=0)
          valid=0;
      }
    }
    i+=2;
  }
  return valid;
}

void generate(int n,int p,int **pieces,int level,int *solution,int *value,int *cb,int *valid)
{
  if(solution[level]!=-1 && *valid)
  {
    removepiece(pieces[level],level,solution,n,cb);
    *value+=pieces[level][0];
  }
  solution[level]++;
  if(solution[level]==n*n)
  {
    *valid=1;
  }
  else
  {
    *valid=feasiblepiece(pieces[level],level,solution,n,cb);
    if(*valid)
    {
      putpiece(pieces[level],level,solution,n,cb);
      *value-=pieces[level][0];
    }
  }
}

int criterion(int p,int level,int valid)
{
  return(level!=p-1 && valid);
}

int morebrothers(int n,int nivel,int *solution)
{
  return(solution[nivel]<n*n);
}

void moveback(int n,int p,int **pieces,int level,int *solution,int *value,int *cb,int *valid)
{
  if(solution[level]!=n*n && *valid)
  {
    removepiece(pieces[level],level,solution,n,cb);
    *value+=pieces[level][0];
  }
  *valid=1;
  solution[level]=-1;
}

void writeall(int n,int p,int level,int valid,int value,int VOA,int *solution,int *cb)
{
  int i,j;

  printf("Level %d, value %d, optimum %d\n valid %d, solution:\n     ",level,value,VOA,valid);
  for(i=0;i<p;i++)
  {
    printf("(%d,%d) ",solution[i]/n,solution[i]%n);
  }
  printf("\n");
  printf("Checkerboard:\n");
  for(i=0;i<n;i++)
  {
    for(j=0;j<n;j++)
    {
        printf("%3d ",cb[i*n+j]);
    }
    printf("\n");
  }
  printf("\n");
}


int sec(int n,int p,int **pieces,int node,int np)
{
  int i,valid=1,value=n*n,VOA=n*n,level=0,*cb,*solution;
  char c;

#ifdef DEBUG
  printf("entra\n");
  scanf("%c\n",&c);
#endif
  cb=(int *) calloc(sizeof(int),n*n); //space for the checkerboard
  solution=(int *) malloc(sizeof(int)*p); //solution[i] is the position of the checkerboard where piece i is installed
  for(i=0;i<p;i++)
  {
    solution[i]=-1; //indicates no decision on piece i is taken
  }

#ifdef DEBUG
  printf("After Initialization\n");
  writeall(n,p,level,valid,value,VOA,solution,cb);
  scanf("%c\n",&c);
#endif
  do{
    generate(n,p,pieces,level,solution,&value,cb,&valid);
#ifdef DEBUG
  printf("After Generate\n");
  writeall(n,p,level,valid,value,VOA,solution,cb);
 scanf("%c\n",&c);
#endif
    if(valid && value<VOA)
    {
       VOA=value;
#ifdef IMPROVE
  printf("After improving solution, the optimum is %d\n",VOA);
  writeall(n,p,level,valid,value,VOA,solution,cb);
 //scanf("%c\n",&c);
#endif    
    }
    if(criterion(p,level,valid))
    {
#ifdef DEBUG
  printf("The criterion is valid\n");
 // writeall(n,p,level,valid,value,VOA,solution,cb);
 scanf("%c\n",&c);
#endif
      level++;
    }
    else
    {
      while(level!=-1 && morebrothers(n,level,solution)==0)
      {
#ifdef DEBUG
  printf("After Morebrothers\n");
 // writeall(n,p,level,valid,value,VOA,solution,cb);
 scanf("%c\n",&c);
#endif
        moveback(n,p,pieces,level,solution,&value,cb,&valid);
        level--;
#ifdef DEBUG
  printf("After Moveback\n");
 writeall(n,p,level,valid,value,VOA,solution,cb);
 scanf("%c\n",&c);
#endif
      }
    }
  } while(level!=-1);

  free(solution);
  free(cb);
  return VOA;
}
